package org.eclipse.jface.eclipsecon.databinding;

import org.eclipse.core.databinding.DataBindingContext;
import org.eclipse.core.databinding.beans.BeansObservables;
import org.eclipse.core.databinding.observable.Realm;
import org.eclipse.jface.databinding.swt.SWTObservables;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Text;
import org.eclipse.ui.part.ViewPart;

public class View extends ViewPart {
	public static final String ID = "org.eclipse.jface.eclipsecon.databinding.view";
	
	public View() {
		viewModel = new ViewModel();
	}
	
	private ViewModel viewModel;
	private Text name;
	private Text address;
	private Text city;
	private Text state;
	private Text zip;

	public void createPartControl(Composite parent) {
		parent.setLayout(new GridLayout(2, false));
		new Label(parent, SWT.NULL).setText("Contact Information:");
		name = new Text(parent, SWT.BORDER);
		name.setLayoutData(fillSpanHorizontal());
		address = new Text(parent, SWT.BORDER);
		address.setLayoutData(fillSpanHorizontal());
		city = new Text(parent, SWT.BORDER);
		city.setLayoutData(fillGrabHorizontal());
		state = new Text(parent, SWT.BORDER);
		state.setLayoutData(fillGrabHorizontal());
		zip = new Text(parent, SWT.BORDER);
		zip.setLayoutData(fillSpanHorizontal());
		
		Realm.runWithDefault(SWTObservables.getRealm(parent.getDisplay()), new Runnable() {
			public void run() {
				bindGUI(DataBindingContextFactory.create());
			}
		});
	}

	protected void bindGUI(DataBindingContext bindingContext) {
		final Person person = viewModel.getContact();

		bindingContext.bindValue(SWTObservables.observeText(name, SWT.FocusOut),
				BeansObservables.observeValue(person, "name"), null);
		bindingContext.bindValue(SWTObservables.observeText(address, SWT.FocusOut),
				BeansObservables.observeValue(person, "address"), null);
		bindingContext.bindValue(SWTObservables.observeText(city, SWT.FocusOut),
				BeansObservables.observeValue(person, "city"), null);
		bindingContext.bindValue(SWTObservables.observeText(state, SWT.FocusOut),
				BeansObservables.observeValue(person, "state"), null);
		bindingContext.bindValue(SWTObservables.observeText(zip, SWT.FocusOut),
				BeansObservables.observeValue(person, "zip"), null);
	}

	private Object fillGrabHorizontal() {
		return new GridData(SWT.FILL, SWT.NONE, true, false);
	}

	private Object fillSpanHorizontal() {
		GridData gd = new GridData(SWT.FILL, SWT.NONE, true, false);
		gd.horizontalSpan = 2;
		return gd;
	}

	/**
	 * Passing the focus request to the viewer's control.
	 */
	public void setFocus() {
		name.setFocus();
	}
}